@@ -239,14 +239,16 @@ emerillon_sidebar_menu_position_under (GtkMenu *menu,
   GtkWidget *widget;
 
   g_return_if_fail (GTK_IS_BUTTON (user_data));
-  g_return_if_fail (GTK_WIDGET_NO_WINDOW (user_data));
-
+  g_return_if_fail (!gtk_widget_get_has_window (GTK_WIDGET (user_data)));
+  
   widget = GTK_WIDGET (user_data);
 
-  gdk_window_get_origin (widget->window, x, y);
+  gdk_window_get_origin (gtk_widget_get_window (widget), x, y);
 
-  *x += widget->allocation.x;
-  *y += widget->allocation.y + widget->allocation.height;
+  GtkAllocation allocation;
+  gtk_widget_get_allocation (widget, &allocation);
+  *x += allocation.x;
+  *y += allocation.y + allocation.height;
 
   *push_in = FALSE;
 }
@@ -263,7 +265,9 @@ emerillon_sidebar_select_button_press_cb (GtkWidget *widget,
       GtkRequisition requisition;
       gint width;
 
-      width = widget->allocation.width;
+      GtkAllocation allocation;
+      gtk_widget_get_allocation (widget, &allocation);
+      width = allocation.width;
 
       gtk_widget_set_size_request (sidebar->priv->menu, -1, -1);
       gtk_widget_size_request (sidebar->priv->menu, &requisition);
@@ -399,6 +403,7 @@ emerillon_sidebar_init (EmerillonSidebar *sidebar)
   sidebar->priv->select_button = gtk_toggle_button_new ();
   gtk_button_set_relief (GTK_BUTTON (sidebar->priv->select_button), 
       GTK_RELIEF_NONE);
+  gtk_widget_set_sensitive (sidebar->priv->select_button, FALSE);
 
   g_signal_connect (sidebar->priv->select_button, "button_press_event",
       G_CALLBACK (emerillon_sidebar_select_button_press_cb),
@@ -481,6 +486,16 @@ emerillon_sidebar_new (void)
   return sidebar;
 }
 
+/**
+ * emerillon_sidebar_add_page:
+ * @sidebar: (in): An #EmerillonSidebar instance
+ * @title: (in): Sidebar title
+ * @main_widget: (in): A #GtkWidget which should be embedded in the sidebar
+ *
+ * Adds a new page to the sidebar
+ *
+ * See: emerillon_window_get_sidebar()
+ */
 void
 emerillon_sidebar_add_page (EmerillonSidebar *sidebar,
                              const gchar *title,
@@ -538,6 +553,9 @@ emerillon_sidebar_add_page (EmerillonSidebar *sidebar,
 
   g_free (label_title);
 
+  if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (sidebar->priv->notebook)) > 1)
+    gtk_widget_set_sensitive (sidebar->priv->select_button, TRUE);
+
   g_signal_emit (G_OBJECT (sidebar), signals[SIGNAL_PAGE_ADDED],
       0, main_widget);
 }
@@ -549,6 +567,7 @@ emerillon_sidebar_remove_page (EmerillonSidebar *sidebar,
   GtkTreeIter iter;
   GtkWidget *widget, *menu_item;
   gboolean valid;
+  gboolean visible;
   gint index;
 
   g_return_if_fail (EMERILLON_IS_SIDEBAR (sidebar));
@@ -575,6 +594,9 @@ emerillon_sidebar_remove_page (EmerillonSidebar *sidebar,
 
   if (valid)
     {
+      visible = (index == gtk_notebook_get_current_page (
+          GTK_NOTEBOOK (sidebar->priv->notebook)));
+
       gtk_notebook_remove_page (GTK_NOTEBOOK (sidebar->priv->notebook), 
           index);
 
@@ -583,11 +605,36 @@ emerillon_sidebar_remove_page (EmerillonSidebar *sidebar,
       gtk_list_store_remove (GTK_LIST_STORE (sidebar->priv->page_model), 
           &iter);
 
-      g_signal_emit (G_OBJECT (sidebar), 
+      /* If we removed the visible tab, we select the first page or remove
+       * the text in the dropdown menu if no page is left */
+      if (visible)
+        {
+          if (!emerillon_sidebar_is_empty(sidebar))
+            {
+              valid = gtk_tree_model_get_iter_first (sidebar->priv->page_model, &iter);
+              emerillon_sidebar_select_page (sidebar, &iter);
+            }
+          else
+            gtk_label_set_text (GTK_LABEL (sidebar->priv->label), "");
+        }
+
+      if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (sidebar->priv->notebook)) <= 1)
+        gtk_widget_set_sensitive (sidebar->priv->select_button, FALSE);
+
+      g_signal_emit (G_OBJECT (sidebar),
           signals[SIGNAL_PAGE_REMOVED], 0, main_widget);
     }
 }
 
+/**
+ * emerillon_sidebar_get_n_pages:
+ * @sidebar: (in): An #EmerillonSidebar instance
+ *
+ * Returns the number of pages @sidebar has.
+ *
+ * Return value: The number of pages @sidebar has. If @sidebar isn't
+ * an #EmerillonSidebar instance this function returns #TRUE.
+ */
 gint
 emerillon_sidebar_get_n_pages (EmerillonSidebar *sidebar)
 {
@@ -597,6 +644,15 @@ emerillon_sidebar_get_n_pages (EmerillonSidebar *sidebar)
       GTK_TREE_MODEL (sidebar->priv->page_model), NULL);
 }
 
+/**
+ * emerillon_sidebar_is_empty:
+ * @sidebar: (in): An #EmerillonSidebar instance
+ *
+ * Checks if the @sidebar is empty
+ *
+ * Return value: #TRUE if @sidebar has no pages or isn't
+ * an #EmerillonSidebar instance
+ */
 gboolean
 emerillon_sidebar_is_empty (EmerillonSidebar *sidebar)
 {
