@@ -34,7 +34,6 @@
 #include <gtk/gtk.h>
 
 #include "config-keys.h"
-#include "../cut-paste/ephy-spinner.h"
 #include "sidebar.h"
 
 static GtkWidget *default_window = NULL;
@@ -61,10 +60,73 @@ struct _EmerillonWindowPrivate
 
   GConfClient *client;
 
-  guint tooltip_messase_context_id;
+
+  guint tooltip_message_context_id;
+
+  /* Defines whether the view position should be updated based on geoclue information */
+  gboolean position_auto_update;
+
+  GeoclueMasterClient *geoclue_client;
+  GeocluePosition *geoclue_position;
+};
+
+enum
+{
+  PROP_0,
+
+  PROP_AUTO_UPDATE
 };
 
 static void     build_ui        (EmerillonWindow *self);
+static void
+position_changed_cb (GeocluePosition *position,
+                     GeocluePositionFields fields,
+                     int timestamp,
+                     double latitude,
+                     double longitude,
+                     double altitude,
+                     GeoclueAccuracy *accuracy,
+                     GError *error,
+                     EmerillonWindow *self);
+
+static void
+emerillon_window_set_property (GObject      *object,
+                               guint         property_id,
+                               const GValue *value,
+                               GParamSpec   *pspec)
+{
+  EmerillonWindow *self = EMERILLON_WINDOW (object);
+  switch (property_id)
+  {
+    case PROP_AUTO_UPDATE:
+      self->priv->position_auto_update = g_value_get_boolean(value);
+      if (self->priv->geoclue_position != NULL)
+        geoclue_position_get_position_async (self->priv->geoclue_position,
+                                             (GeocluePositionCallback)position_changed_cb, self);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      break;
+  }
+}
+
+static void
+emerillon_window_get_property (GObject    *object,
+                               guint       property_id,
+                               GValue     *value,
+                               GParamSpec *pspec)
+{
+  EmerillonWindow *self = EMERILLON_WINDOW (object);
+  switch (property_id)
+  {
+    case PROP_AUTO_UPDATE:
+      g_value_set_boolean (value, self->priv->position_auto_update);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      break;
+  }
+}
 
 static gboolean
 set_zoom_for_accuracy (EmerillonWindow *self,
@@ -119,32 +181,31 @@ position_changed_cb (GeocluePosition *position,
     {
       g_printerr ("Error retrieving the current position: %s\n", error->message);
       g_error_free (error);
-      g_object_unref (position);
       return;
     }
   else if (fields & GEOCLUE_POSITION_FIELDS_LATITUDE &&
-           fields & GEOCLUE_POSITION_FIELDS_LONGITUDE)
+           fields & GEOCLUE_POSITION_FIELDS_LONGITUDE && self->priv->position_auto_update)
     {
       /* FIXME: if the next calls are inverted then the wrong position is
        * shown (libchamplain bug). */
       set_zoom_for_accuracy (self, accuracy);
       champlain_view_center_on (self->priv->view, latitude, longitude);
     }
-
-  g_object_unref (g_object_get_data (G_OBJECT (position), "client"));
-  g_object_unref (position);
 }
 
 static void
 emerillon_window_init (EmerillonWindow *self)
 {
   GdkGeometry geometry;
-  GeoclueMaster *master;
-  GeoclueMasterClient *client;
-  GeocluePosition *position;
+  GeoclueMaster *master = NULL;
+  GError *error = NULL;
+
+  gint width, height;
 
   self->priv = EMERILLON_WINDOW_GET_PRIVATE (self);
 
+  self->priv->position_auto_update = FALSE;
+
   /* GConf. */
   self->priv->client = gconf_client_get_default ();
 
@@ -157,33 +218,53 @@ emerillon_window_init (EmerillonWindow *self)
   gtk_window_set_geometry_hints (GTK_WINDOW (self), GTK_WIDGET (self),
       &geometry,GDK_HINT_MIN_SIZE);
 
-  gtk_window_set_default_size (GTK_WINDOW (self), 640, 450);
+  /* Set the window size */
+  width = gconf_client_get_int (self->priv->client,
+      EMERILLON_CONF_UI_WINDOW_WIDTH, NULL);
+  height = gconf_client_get_int (self->priv->client,
+      EMERILLON_CONF_UI_WINDOW_HEIGHT, NULL);
+
+  if (width > 0 && height > 0)
+    gtk_window_set_default_size (GTK_WINDOW (self), width, height);
+  else
+    gtk_window_set_default_size (GTK_WINDOW (self), 640, 450);
 
   /* Current position. */
   master = geoclue_master_get_default ();
-  client = geoclue_master_create_client (master, NULL, NULL);
+  self->priv->geoclue_client = geoclue_master_create_client (master, NULL, &error);
+  if (!self->priv->geoclue_client)
+    {
+       g_warning ("Creating Geoclue Master client failed: %s", error->message);
+       g_error_free (error);
+       g_object_unref (master);
+       return;
+    }
   g_object_unref (master);
 
-  geoclue_master_client_set_requirements (client,
-      GEOCLUE_ACCURACY_LEVEL_COUNTRY, 0, FALSE, GEOCLUE_RESOURCE_ALL, NULL);
-  position = geoclue_master_client_create_position (client, NULL);
-  if (position)
-    {
-      g_object_set_data (G_OBJECT (position), "client", client);
-      geoclue_position_get_position_async (position,
-          (GeocluePositionCallback)position_changed_cb, self);
-    }
+  geoclue_master_client_set_requirements (self->priv->geoclue_client,
+                                          GEOCLUE_ACCURACY_LEVEL_COUNTRY, 0, FALSE, GEOCLUE_RESOURCE_ALL, NULL);
+  self->priv->geoclue_position = geoclue_master_client_create_position (self->priv->geoclue_client, &error);
+  if (self->priv->geoclue_position)
+  {
+    g_object_set_data (G_OBJECT (self->priv->geoclue_position), "client", self->priv->geoclue_client);
+    g_signal_connect(self->priv->geoclue_position, "position-changed",
+                     G_CALLBACK(position_changed_cb), NULL);
+  }
   else
-    {
-      g_object_unref (client);
-      g_object_unref (position);
-    }
+  {
+    g_warning ("Getting Geoclue Position Failed: %s", error->message);
+    g_error_free (error);
+    if (self->priv->geoclue_client)
+      g_object_unref (self->priv->geoclue_client);
+  }
+
 }
 
 static void
 emerillon_window_dispose (GObject *object)
 {
   EmerillonWindow *self = EMERILLON_WINDOW (object);
+  gint width, height;
 
   if (self->priv->main_actions != NULL)
     {
@@ -192,10 +273,28 @@ emerillon_window_dispose (GObject *object)
     }
 
   if (self->priv->client)
+  {
+    /* Save the window size */
+    gtk_window_get_size (GTK_WINDOW (self), &width, &height);
+    width = gconf_client_set_int (self->priv->client,
+                                  EMERILLON_CONF_UI_WINDOW_WIDTH, width, NULL);
+    height = gconf_client_set_int (self->priv->client,
+                                   EMERILLON_CONF_UI_WINDOW_HEIGHT, height, NULL);
+
+    gconf_client_remove_dir (self->priv->client, EMERILLON_CONF_DIR, NULL);
+    g_object_unref (self->priv->client);
+    self->priv->client = NULL;
+  }
+
+  if (self->priv->geoclue_client)
     {
-      gconf_client_remove_dir (self->priv->client, EMERILLON_CONF_DIR, NULL);
-      g_object_unref (self->priv->client);
-      self->priv->client = NULL;
+       g_object_unref (self->priv->geoclue_client);
+       self->priv->geoclue_client = NULL;
+    }
+   if (self->priv->geoclue_position)
+    {
+       g_object_unref (self->priv->geoclue_position);
+       self->priv->geoclue_client = NULL;
     }
 
   G_OBJECT_CLASS (emerillon_window_parent_class)->dispose (object);
@@ -241,6 +340,23 @@ emerillon_window_class_init (EmerillonWindowClass *klass)
   object_class->dispose = emerillon_window_dispose;
   object_class->finalize = emerillon_window_finalize;
 
+  /* Initialize object properties */
+  object_class->get_property = emerillon_window_get_property;
+  object_class->set_property = emerillon_window_set_property;
+
+  /**
+   * EmerillonWindow:auto-update:
+   *
+   * Toggle automatic update of the map position based on geoclue data.
+   * When enabled also gets the current position.
+   *
+   * Since 0.1.2
+   */
+  g_object_class_install_property(object_class, PROP_AUTO_UPDATE,
+                                  g_param_spec_boolean ("auto-update", "Position auto update",
+                                "Toggle automatic update of the map position based on geoclue data",
+                                FALSE, G_PARAM_READWRITE));
+
   g_type_class_add_private (object_class, sizeof (EmerillonWindowPrivate));
 }
 
@@ -261,9 +377,9 @@ state_changed_cb (GtkWidget *widget,
 
   g_object_get (self->priv->view, "state", &state, NULL);
   if (state == CHAMPLAIN_STATE_LOADING)
-    ephy_spinner_start (EPHY_SPINNER (self->priv->throbber));
+    gtk_spinner_start (GTK_SPINNER (self->priv->throbber));
   else
-    ephy_spinner_stop (EPHY_SPINNER (self->priv->throbber));
+    gtk_spinner_stop (GTK_SPINNER (self->priv->throbber));
 }
 
 static void
@@ -279,10 +395,8 @@ zoom_changed_cb (GtkWidget *widget,
   ChamplainMapSource *source = NULL;
 
   source = champlain_view_get_map_source (self->priv->view);
-  g_object_get (G_OBJECT (source),
-      "min-zoom-level", &min_zoom_level,
-      "max-zoom-level", &max_zoom_level,
-      NULL);
+  min_zoom_level = champlain_map_source_get_min_zoom_level (source);
+  max_zoom_level = champlain_map_source_get_max_zoom_level (source);
 
   zoom_in_action = gtk_action_group_get_action (self->priv->main_actions,
         "ViewZoomIn");
@@ -344,7 +458,16 @@ static void
 cmd_help (GtkAction *action,
           EmerillonWindow *self)
 {
-  g_printerr ("Sorry, help not available\n");
+  GError *error = NULL;
+
+  gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (self)), "ghelp:emerillon",
+                gtk_get_current_event_time (), &error);
+
+  if (error != NULL)
+  {
+    g_printerr ("Error opening help: %s\n", error->message);
+    g_error_free (error);
+  }
 }
 
 static void
@@ -354,6 +477,7 @@ cmd_about (GtkAction *action,
   static const char *authors[] = {
       "Marco Barisione <marco@barisione.org>",
       "Pierre-Luc Beaudoin <pierre-luc.beaudoin@novopia.com>",
+      "Simon Wenner <simon@wenner.ch>",
       NULL
   };
   static const char *documenters[] = {
@@ -383,12 +507,13 @@ cmd_about (GtkAction *action,
       "program-name", _("Emerillon"),
       "version", VERSION,
       "copyright", "Copyright \xc2\xa9 2008 Marco Barisione\n"
-                   "Copyright \xc2\xa9 2009 Novopia Inc.",
+                   "Copyright \xc2\xa9 2009 Novopia Inc.\n"
+                   "Copyright \xc2\xa9 2010 Emerillon Contributors.",
       "comments",_("A map viewer for the GNOME desktop"),
       "authors", authors,
       "documenters", documenters,
       "translator-credits", translators,
-      "website", "http://www.novopia.com/emerillon",
+      "website", "http://projects.gnome.org/emerillon",
       //"logo-icon-name", "emerillon",
       "license", license_translated,
       "wrap-license", TRUE,
@@ -413,19 +538,19 @@ cmd_map_change_map (GtkRadioAction *action,
     {
       default:
       case 0:
-        source = champlain_map_source_factory_create (factory,
+        source = champlain_map_source_factory_create_cached_source (factory,
             CHAMPLAIN_MAP_SOURCE_OSM_MAPNIK);
         break;
       case 1:
-        source = champlain_map_source_factory_create (factory,
+        source = champlain_map_source_factory_create_cached_source (factory,
             CHAMPLAIN_MAP_SOURCE_OSM_CYCLE_MAP);
         break;
       case 2:
-        source = champlain_map_source_factory_create (factory,
+        source = champlain_map_source_factory_create_cached_source (factory,
             CHAMPLAIN_MAP_SOURCE_OSM_TRANSPORT_MAP);
         break;
       case 3:
-        source = champlain_map_source_factory_create (factory,
+        source = champlain_map_source_factory_create_cached_source (factory,
             CHAMPLAIN_MAP_SOURCE_MFF_RELIEF);
         break;
 
@@ -543,7 +668,7 @@ menu_item_select_cb (GtkMenuItem *proxy,
   if (tooltip)
     {
       gtk_statusbar_push (GTK_STATUSBAR (self->priv->statusbar),
-          self->priv->tooltip_messase_context_id, tooltip);
+          self->priv->tooltip_message_context_id, tooltip);
       g_free (tooltip);
     }
 }
@@ -553,7 +678,7 @@ menu_item_deselect_cb (GtkMenuItem *proxy,
                        EmerillonWindow *self)
 {
   gtk_statusbar_pop (GTK_STATUSBAR (self->priv->statusbar),
-      self->priv->tooltip_messase_context_id);
+      self->priv->tooltip_message_context_id);
 }
 
 static void
@@ -593,7 +718,7 @@ sidebar_visibility_changed_cb (GtkWidget *widget,
   GtkAction *action;
   gboolean visible;
 
-  visible = GTK_WIDGET_VISIBLE (self->priv->sidebar);
+  visible = gtk_widget_get_visible (self->priv->sidebar);
 
   gconf_client_set_bool (self->priv->client, EMERILLON_CONF_UI_SIDEBAR,
       visible, NULL);
@@ -731,9 +856,7 @@ build_ui (EmerillonWindow *self)
   self->priv->toolbar = gtk_ui_manager_get_widget (self->priv->ui_manager,
       "/Toolbar");
 
-  self->priv->throbber = ephy_spinner_new ();
-  ephy_spinner_set_size (EPHY_SPINNER (self->priv->throbber),
-      GTK_ICON_SIZE_LARGE_TOOLBAR);
+  self->priv->throbber = gtk_spinner_new ();
 
   throbber = gtk_tool_item_new ();
   gtk_container_add (GTK_CONTAINER (throbber), self->priv->throbber);
@@ -752,7 +875,7 @@ build_ui (EmerillonWindow *self)
       GTK_WIDGET (self->priv->statusbar), FALSE, FALSE, 0);
   gtk_widget_show (self->priv->statusbar);
 
-  self->priv->tooltip_messase_context_id = gtk_statusbar_get_context_id (
+  self->priv->tooltip_message_context_id = gtk_statusbar_get_context_id (
       GTK_STATUSBAR (self->priv->statusbar), "tooltip-message");
 
   /* Viewport. */
@@ -784,11 +907,6 @@ build_ui (EmerillonWindow *self)
   self->priv->sidebar = emerillon_sidebar_new ();
   gtk_widget_set_size_request (self->priv->sidebar, 200, -1);
 
-  g_signal_connect_after (self->priv->sidebar, "show",
-      G_CALLBACK (sidebar_visibility_changed_cb), self);
-  g_signal_connect_after (self->priv->sidebar, "hide",
-      G_CALLBACK (sidebar_visibility_changed_cb), self);
-
   /* Horizontal pane. */
   hpaned = gtk_hpaned_new ();
   gtk_paned_pack1 (GTK_PANED (hpaned), self->priv->sidebar, FALSE, FALSE);
@@ -796,6 +914,11 @@ build_ui (EmerillonWindow *self)
   gtk_widget_show (self->priv->sidebar);
   gtk_widget_show (viewport);
 
+  g_signal_connect_after (self->priv->sidebar, "show",
+      G_CALLBACK (sidebar_visibility_changed_cb), self);
+  g_signal_connect_after (self->priv->sidebar, "hide",
+      G_CALLBACK (sidebar_visibility_changed_cb), self);
+
   gtk_box_pack_start (GTK_BOX (vbox), hpaned, TRUE, TRUE, 0);
   gtk_widget_show (hpaned);
 
@@ -818,6 +941,14 @@ emerillon_window_get_ui_manager (EmerillonWindow *window)
   return window->priv->ui_manager;
 }
 
+/**
+ * emerillon_window_get_toolbar:
+ * @window: (in): An #EmerillonWindow instance
+ *
+ * Retrieves Emerillon's toolbar
+ *
+ * Return value: A #GtkWidget containig Emerillon's toolbar
+ */
 GtkWidget *
 emerillon_window_get_toolbar (EmerillonWindow *window)
 {
@@ -826,6 +957,14 @@ emerillon_window_get_toolbar (EmerillonWindow *window)
   return window->priv->toolbar;
 }
 
+/**
+ * emerillon_window_get_sidebar:
+ * @window: (in): An #EmerillonWindow instance
+ *
+ * Retrieves Emerillon's sidebar
+ *
+ * Return value: A #GtkWidget containig the current sidebar
+ */
 GtkWidget *
 emerillon_window_get_sidebar (EmerillonWindow *window)
 {
