@@ -47,9 +47,14 @@ struct _PlacemarksPluginPrivate
   GtkTreeModel *model;
   GtkWidget *menu;
 
+  /** Layer with placemark markers */
+  ChamplainLayer *markers_layer;
   guint deleted_cb_id;
 };
 
+/**
+ * Go to the specified placemark
+ */
 static void
 go_cb (GtkAction *action,
        PlacemarksPlugin *plugin)
@@ -95,6 +100,27 @@ go_cb (GtkAction *action,
   champlain_view_center_on (priv->map_view, lat, lon);
 }
 
+static ChamplainMarker *
+add_marker(PlacemarksPlugin *plugin, const gchar *name, gdouble lat, gdouble lon)
+{
+  PlacemarksPluginPrivate *priv;
+  priv = PLACEMARKS_PLUGIN (plugin)->priv;
+  ChamplainMarker *marker;
+
+  ClutterColor orange = { 0xf3, 0x94, 0x07, 0xbb };
+  marker = CHAMPLAIN_MARKER ( champlain_marker_new_with_text (name, "Serif 14",
+                                                              NULL, NULL));
+  champlain_marker_set_use_markup (CHAMPLAIN_MARKER (marker), TRUE);
+  champlain_marker_set_alignment (CHAMPLAIN_MARKER (marker), PANGO_ALIGN_RIGHT);
+  champlain_marker_set_color (CHAMPLAIN_MARKER (marker), &orange);
+
+  champlain_base_marker_set_position (CHAMPLAIN_BASE_MARKER (marker),
+                                      lat, lon);
+  champlain_layer_add_marker (priv->markers_layer, CHAMPLAIN_BASE_MARKER (marker));
+
+  return marker;
+}
+
 static guint
 append_menu_item (PlacemarksPlugin *plugin,
                   const gchar *id,
@@ -243,12 +269,14 @@ add_placemark (PlacemarksPlugin *plugin,
   gchar *lat_str, *lon_str, *zoom_str;
   GtkTreeIter iter;
   PlacemarksPluginPrivate *priv;
+  ChamplainMarker *marker;
 
   priv = PLACEMARKS_PLUGIN (plugin)->priv;
 
   lat_str = g_strdup_printf ("%f", lat);
   lon_str = g_strdup_printf ("%f", lon);
   zoom_str = g_strdup_printf ("%d", zoom);
+  marker = add_marker (plugin, name, lat, lon);
 
   gtk_list_store_append (GTK_LIST_STORE (priv->model), &iter);
   gtk_list_store_set (GTK_LIST_STORE (priv->model), &iter,
@@ -260,6 +288,7 @@ add_placemark (PlacemarksPlugin *plugin,
                       COL_LON_STR, lon_str,
                       COL_ZOOM, zoom,
                       COL_ZOOM_STR, zoom_str,
+                      COL_MARKER, marker,
                       -1);
 
   g_free (lat_str);
@@ -370,6 +399,7 @@ load_placemarks (PlacemarksPlugin *plugin)
   groups = g_key_file_get_groups (file, &group_count);
   priv->placemark_count = group_count;
 
+
   for (i = 0; i < group_count; i++)
     {
       gchar *name;
@@ -531,6 +561,9 @@ activated (EthosPlugin *plugin)
   priv->window = EMERILLON_WINDOW (emerillon_window_dup_default ());
   priv->map_view = emerillon_window_get_map_view (priv->window);
 
+  priv->markers_layer = champlain_selection_layer_new ();
+  champlain_view_add_layer(priv->map_view, priv->markers_layer);
+
   manager = emerillon_window_get_ui_manager (priv->window);
 
   priv->action_group = gtk_action_group_new ("PlacemarksActions");
@@ -558,7 +591,8 @@ activated (EthosPlugin *plugin)
                               G_TYPE_STRING,       /* Longitude as a string */
                               G_TYPE_INT,          /* Zoom level */
                               G_TYPE_STRING,       /* Zoom level as a string */
-                              G_TYPE_UINT);        /* UI ID */
+                              G_TYPE_UINT,        /* UI ID */
+                              G_TYPE_POINTER);       /* Marker pointer */
   priv->model = GTK_TREE_MODEL (store);
   priv->deleted_cb_id  =  g_signal_connect (priv->model,
                                             "row-deleted",
