@@ -119,10 +119,10 @@ result_cb (RestProxyCall *call,
   n = rest_xml_node_find (root, "geoname");
   i = 1;
 
-  min_lat = CHAMPLAIN_MAX_LAT;
-  max_lat = CHAMPLAIN_MIN_LAT;
-  min_lon = CHAMPLAIN_MAX_LONG;
-  max_lon = CHAMPLAIN_MIN_LONG;
+  min_lat = 90;
+  max_lat = -90;
+  min_lon = 180;
+  max_lon = -180;
 
   while (n)
     {
@@ -493,7 +493,7 @@ activated (EthosPlugin *plugin)
   g_object_set (cell,
                 "ellipsize", PANGO_ELLIPSIZE_END,
                 NULL);
-
+  /* Translators: In this case "No" means "number". */
   column = gtk_tree_view_column_new_with_attributes (_("No"),
                                                      cell,
                                                      "text", COL_SYMBOL,
